<?php

namespace Concrete\Package\KoubaAreasCleanup\Controller\SinglePage\Dashboard\Blocks;

use Concrete\Core\Block\Block;
use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Support\Facade\Database;
use Concrete\Core\Page\Page;
use Concrete\Core\Area\Area;
use stdClass;
use Concrete\Core\Application\EditResponse;
use Concrete\Core\Page\Stack\Pile\Pile;
use Concrete\Core\Support\Facade\Log;

defined('C5_EXECUTE') or die("Access Denied.");
class AreasCleanup extends DashboardPageController
{
  public function view($cID = false)
  {
    $this->requireAsset('jquery/ui');
    $this->requireAsset('fancytree');
    $this->set('treeViewData', $this->getTreeViewData());
  }

  private function getAreaBlockCount($areaHandle, $pageID, $activePageVersionID)
  {
    $db = Database::connection();
    $result = $db->GetRow("select
                              coalesce(sum(case  when cvID = ? then 1 else 0 end ),0) as activeCount,
                              count(distinct bID) as total
                            from CollectionVersionBlocks
                            where arHandle = ?
                              and cID = ?
                              and bID is not null ", [$activePageVersionID, $areaHandle, $pageID]);

    if ($result) {
      return ['activeCount' => $result['activeCount'], 'total' => $result['total']];
    } else {
      return false;
    }
  }


  private function getTreeViewData()
  {
    $navigationHelper = $this->app->make('helper/navigation');

    $db = Database::connection();
    $data = $db->GetAll('SELECT arID, arHandle, cID, arIsGlobal, arParentID FROM Areas ORDER BY arHandle');

    $topAreas = array_filter($data, function ($a) {
      return $a['arParentID'] == 0;
    });

    $areas = [];

    foreach ($topAreas as $ta) {
      $arHandle = $ta['arHandle'];
      if (!array_key_exists($arHandle, $areas)) {

        $areas[$arHandle] = [
            'isGlobal' => $ta['arIsGlobal']
          , 'totalBlocks' => 0
          , 'arIsGlobal' => $ta['arIsGlobal']
          , 'pages' => []
        ];
      }

      $walk = function ($ta)
      use (&$walk, &$areas, &$arHandle, &$navigationHelper, &$data) {
        $cID = $ta['cID'];

        $page = Page::getByID($cID, 'ACTIVE');
        $activePage = Page::getByID($cID, 'ACTIVE');
        $activePageVersionID = $activePage->getVersionID();
        $recentPage = Page::getByID($cID, 'RECENT');
        $recentPageVersionID = $recentPage->getVersionID();
        $pageHasNewerVersion = $recentPageVersionID > $activePageVersionID;
        $blocksCount = $this->getAreaBlockCount($ta['arHandle'], $ta['cID'], $activePageVersionID);
        $url = $navigationHelper->getLinkToCollection($activePage, true);

        if (!array_key_exists($cID, $areas[$arHandle]['pages'])) {

          $areas[$arHandle]['pages'][$cID] = [
            'cID' => $cID
            , 'activePageBlocks' => 0
            , 'url' => $url
            , 'name' => $page->getCollectionName()
            , 'hasNewerVersion' => $pageHasNewerVersion
          ];
        }
        $areas[$arHandle]['pages'][$cID]['activePageBlocks'] += $blocksCount['activeCount'];

        $areas[$arHandle]['totalBlocks'] += $blocksCount['activeCount'];

        $subareas = array_filter($data, function ($aItem) use (&$ta) {
          return $aItem['arParentID'] == $ta['arID'];
        });

        if (count($subareas) > 0) {
          foreach ($subareas as $sa) {
            $walk($sa);
          }
        }

      };
      $walk($ta);
    }

    $data = [];
    $counter = 1;
    if (count($areas) > 0) {
      foreach ($areas as $key => $val) {
        $pagesWithBlocksCount = 0;
        $pagesWithBlocksCount = count(array_filter($val['pages'], function($p){
          return $p['activePageBlocks'] > 0;
        }));

        $tmp = new \stdClass();
        $tmp->title = '<span class="area-handle ' . ($val['totalBlocks'] > 0 ? ' bold ' : '')
          . ' " data-area-handle="' . $key . '">'
          . $key . ($val['arIsGlobal'] == true ? ' [' . t('Sitewide') .']' : '') . ' (' .$pagesWithBlocksCount . '/' . t2('%d page', '%d pages', count($val['pages'])) . ')</span>';
        $tmp->key = $counter++;
        $tmp->folder = true;
        $tmp->children = [];
        if (count($val['pages']) > 0) {
          foreach ($val['pages'] as $v) {
            $n = new stdClass();
            $n->title = '<span class="page-name ' . ($v['activePageBlocks'] == 0 ? ' no-active-blocks' : '')
              . '" data-cID="' . $v['cID']
              . '" data-url = "' . $v['url'] . '"'
              . ' data-page-name="' . $v['name'] . '"'
              . ' data-area-handle="' . $key . '"'
              . ' data-blocks-count="' . $v['activePageBlocks'] . '"'
              . '>'
              . (strlen($v['name']) ? $v['name'] : ' (Page has no name or forbidden)') . ' (' . t2('%d block', '%d blocks', $v['activePageBlocks'])  . ')</span>';
            $n->key = $counter++;
            $tmp->children[] = $n;
          }
        }
        $data[] = $tmp;
      }

      return $data;
    }

  }

  public function delete_area()
  {
    $response = new EditResponse();

      $areaHandle = $this->post('area_handle');

      $db = Database::connection();
      $areas = [];
      try {
        $areasData = $db->GetAll('SELECT DISTINCT arID, arHandle FROM Areas where arHandle =?', $areaHandle);

        if (count($areasData) > 0) {

          try {
            foreach ($areasData as $a) {

              $walk = function ($a) use (&$walk, &$db, &$areas) {
                if (!array_key_exists($a['arHandle'], $areas)) {
                  $areas[] = "'" . $a['arHandle'] . "'";
                }
                $subareas = $db->GetAll('SELECT DISTINCT arID, arHandle FROM Areas WHERE arParentID = ?', $a['arID']);
                if (count($subareas) > 0) {
                  foreach ($subareas as $sa) {
                    $walk($sa);
                  }
                }
              };
              $walk($a);
            }
          } catch (\Exception $ex) {
            Log::addEntry('Error ' . $ex->getMessage());
          }
        }
        $areasParam = implode(',', $areas);
        $data = $db->GetAll('SELECT DISTINCT bID,cID,arHandle FROM CollectionVersionBlocks
                           WHERE bID IS NOT NULL AND arHandle IN (' . $areasParam . ')');

        if (count($data) > 0) {
          foreach ($data as $d) {
            $page = Page::getByID($d['cID']);
            $block = Block::getByID($d['bID'], $page, $d['arHandle']);
            if ($block != null) {
              $block->deleteBlock(true);
            }
          }
        }

        $db->executeQuery('DELETE FROM  CollectionVersionBlocks
                         WHERE bID IS NOT NULL AND arHandle IN ('. $areasParam . ')');
        $db->executeQuery('DELETE FROM Areas WHERE arHandle IN ('.$areasParam . ')');

        $response->setMessage(t('Area and blocks are deleted.'));
      } catch (\Exception $ex) {
        $response->setError(t('Error happened during delete.'));
      }

    $response->outputJSON();
  }

  public function area_pages_count($areaHandle = '')
  {
    if (strlen($areaHandle) == 0) {
      $this->app->make('helper/ajax')->sendResult(null);;
    }

    $areas = $this->getAllAreas($areaHandle);
    $tAreas = [];
    foreach ($areas as $a){
      $tAreas[]='"'. $a . '"';
    }

    $db = Database::connection();
    $data = $db->GetRow('SELECT coalesce(sum(case when CV.cvIsApproved = 1 then 1 else 0 end),0) as numberOfApprovedVersions,
                                 coalesce(sum(case when CV.cvIsApproved = 0 then 1 else 0 end),0) as numberOfNotApprovedVersions
                          from ( SELECT distinct T2.cID, T2.cvID FROM Areas T1 
                                 left join CollectionVersionBlocks T2 on T2.arHandle = T1.arHandle
                                 WHERE T2.bID is not null AND T1.arHandle in(' . implode(',', $tAreas) . ') )T
                          left join CollectionVersions CV on T.cID = CV.cID AND T.cvID = CV.cvID');
    $message = '';
    $response = new stdClass();
    if(count($data) > 0){
      $message = t('This will delete all blocks associated with the area %1$s on %2$s published pages and  %3$s unpublished page versions. This cannot be undone.',$areaHandle, $data['numberOfApprovedVersions'], $data['numberOfNotApprovedVersions']);
      $response->numberOfApprovedVersions = $data['numberOfApprovedVersions'];
      $response->numberOfNotApprovedVersions = $data['numberOfNotApprovedVersions'];
    }
    $response->message = $message;
    $this->app->make('helper/ajax')->sendResult($response);
  }

  public function area_preview_html($areaHandle, $pageID)
  {
    $response = "";

    if (strlen($areaHandle) > 0 && intval($pageID) > 0) {
      $area = new Area($areaHandle);
      $page = Page::getByID($pageID);
      $blocks = $area->getAreaBlocksArray($page);
      if (count($blocks) > 0) {
        ob_start();
        print $area->display($page, $blocks);
        $response = ob_get_contents();
        ob_end_clean();
      }
    } else {
      throw new Exception(t('Area is empty for this page.'));
    }
    $this->app->make('helper/ajax')->sendResult(json_encode($response));
  }

  private function getAllAreas($areaHandle = ''){
    $db = Database::connection();
    $areas = [];
    $walk = function ($arHandle)
    use(&$areas, &$db, &$walk){
      $areas[]=$arHandle;
      $handles = $db->GetAll('SELECT DISTINCT T2.arHandle FROM Areas T1 
                               JOIN Areas T2 ON T1.arID = T2.arParentID 
                               WHERE T1.arHandle = ?', [$arHandle]);
      if(count($handles) > 0){
        foreach ($handles as $h){
          $walk($h['arHandle']);
        }
      }
    };
    $walk($areaHandle);

    return $areas;
  }

  public function copy_blocks($areaHandle, $pageID){

    $db = Database::connection();

    $page = Page::getByID($pageID, 'ACTIVE');

    $areas = [];
    $walk = function ($arHandle)
    use(&$areas, &$db, &$walk){
      $areas[]=$arHandle;
       $handles = $db->GetAll('SELECT DISTINCT T2.arHandle FROM Areas T1 
                               JOIN Areas T2 ON T1.arID = T2.arParentID 
                               WHERE T1.arHandle = ?', [$arHandle]);
       if(count($handles) > 0){
         foreach ($handles as $h){
           $walk($h['arHandle']);
         }
       }
    };
    $walk($areaHandle);

    $copiedBlocksCount = 0;

    if(count($areas) > 0){
      foreach ($areas as $arHandle){
        $blocks = $db->GetAll('SELECT DISTINCT bID FROM CollectionVersionBlocks WHERE arHandle = ? AND cID = ? AND cvID = ?', [$arHandle, $pageID, $page->getVersionID()]);
        if(count($blocks) > 0){

          $a = Area::get($page, $arHandle);
          if ($a->isGlobalArea()) {
            $ax = STACKS_AREA_NAME;
            $cx = Stack::getByName($arHandle);
          }
          foreach ($blocks as $t){
            try{

              $bID = $t['bID'];
              $b = Block::getByID($bID, $cx, $ax);
              if($b->getBlockTypeHandle() == BLOCK_HANDLE_LAYOUT_PROXY){
                continue;
              }
              if ($b->getBlockTypeHandle() == BLOCK_HANDLE_SCRAPBOOK_PROXY) {
                $bi = $b->getInstance();
                $b = Block::getByID($bi->getOriginalBlockID());
              }

              $obj = &$b;

              $p = Pile::getDefault();
              $p->add($obj);
              $added = true;
              $copiedBlocksCount++;
            }catch (\Exception $ex){
              Log::addEntry($ex->getMessage() . ' | '. $ex->getFile() . ' | ' . $ex->getTraceAsString());
            }

          }
        }
      }
    }

    $response = new EditResponse();
    $response->setMessage(t2('%d block copied', '%d blocks copied', $copiedBlocksCount));
    $response->outputJSON();

  }
}